package com.ora.jsp.beans.counter;

import java.io.*;
import java.util.*;
import javax.servlet.http.*;
import javax.servlet.jsp.*;

/**
 * This class maintains page counters.
 *
 * Note that this class is intended to be used in session and application
 * scope so regular setter and getter methods for pageContext and 
 * isPermanent can not be used; all info needed to get and increment the
 * counter value must be provided as parameters to the corresponding
 * method to avoid multi-threading issues.
 *
 * @author Hans Bergsten, Gefion software <hans@gefionsoftware.com>
 * @version 1.0
 */
public class CounterBean implements Serializable {
    /* 
     * Hashtable with a URI as key and an int[] as value.
     * An int[] of size 1 is used to hold the counter value
     * instead of an Integer to avoid the cost of instantiation
     * when incrementing
     */
    private Hashtable counters = new Hashtable();

    /**
     * Returns the current value of the counter for this page.
     *
     * @param pageContext the PageContext for the page
     * @return the current counter value, or -1 if it's not found
     * @exception throws Exception if the pageContext is null
     */
    public int getCurrentValue(PageContext pageContext) throws Exception {
        int value = -1;
        if (pageContext == null) {
            throw new Exception("no pageContext");
        }
        HttpServletRequest req = (HttpServletRequest) pageContext.getRequest();
        String uri = req.getRequestURI();
        int[] counter = (int[]) counters.get(uri);
        if (counter != null) {
            value = counter[0];
        }
        return value; 
    }
    
    /**
     * Increments the counter for this page and returns the new value.
     *
     * @param pageContext the PageContext for the page
     * @return the current counter value incremented by one
     * @exception throws Exception if the pageContext is null
     */
    public int getNextValue(PageContext pageContext) throws Exception {
        incrementValue(pageContext, false);
        return getCurrentValue(pageContext);
    }
    
    /**
     * Increments the counter for this page and returns the new value.
     * If permanent is true the new value is also saved to a file.
     *
     * @param pageContext the PageContext for the page
     * @param permanent set to true if the new value should be saved to
     *   a file
     * @return the current counter value incremented by one
     * @exception throws Exception if the pageContext is null
     */
    public int getNextValue(PageContext pageContext, boolean permanent) 
        throws Exception {
        incrementValue(pageContext, permanent);
        return getCurrentValue(pageContext);
    }
    
    /**
     * Increments the counter for this page. If the permanent is true,
     * the new value is also saved in a file.
     *
     * @param pageContext the PageContext for the page
     * @param permanent set to true if the new value should be saved to
     *   a file
     * @exception throws Exception if the pageContext is null
     */
    public void incrementValue(PageContext pageContext, boolean permanent) 
        throws Exception {
        if (pageContext == null) {
            throw new Exception("no pageContext");
        }
        HttpServletRequest req = (HttpServletRequest) pageContext.getRequest();
        String uri = req.getRequestURI();
        /*
         * Synchronized block for incrementing the counter to handle
         * concurrent calls from multiple threads
         */
        synchronized (counters) {
            int[] counter = (int[]) counters.get(uri);
            if (counter == null) {
                counter = new int[1];
                counter[0] = 0;
            }
            counter[0]++;
            counters.put(uri, counter);
        }
    }
}
